<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui version="3" name="KDevHTMLPart" >

<MenuBar>
  <Menu name="file" noMerge="0"><text>&amp;File</text>
    <Action name="print_doc" group="print_merge"/>
  </Menu>
  <Menu name="edit" noMerge="0">
    <Action name="copy_doc_selection" group="edit_paste_merge"/>
  </Menu>
</MenuBar>

<ToolBar name="browserToolBar">
  <Action name="browser_back" group="browser_operations"/>
  <Action name="browser_forward" group="browser_operations"/>
</ToolBar>

</kpartgui>
