

#include "$NEWFILENAMELC$.h"

$NEWCLASS$::$NEWCLASS$(QWidget* parent, const char* name$CAN_BE_MODAL_CPP1$, WFlags fl)
: $BASECLASS$(parent,name$CAN_BE_MODAL_CPP2$,fl)
{
}

$NEWCLASS$::~$NEWCLASS$()
{
}

/*$SPECIALIZATION$*/

