/****************************************************************************
** Form interface generated from reading ui file './implementationwidgetbase.ui'
**
** Created: Wed Apr 30 16:13:25 2008
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef CREATEIMPLEMENATIONWIDGETBASE_H
#define CREATEIMPLEMENATIONWIDGETBASE_H

#include <qvariant.h>
#include <qdialog.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QPushButton;
class QButtonGroup;
class QRadioButton;
class KListView;
class QListViewItem;
class QLabel;
class KLineEdit;

class CreateImplemenationWidgetBase : public QDialog
{
    Q_OBJECT

public:
    CreateImplemenationWidgetBase( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~CreateImplemenationWidgetBase();

    QPushButton* okButton;
    QPushButton* cancelButton;
    QButtonGroup* buttonGroup1;
    QRadioButton* createButton;
    KListView* classView;
    QLabel* classNameLabel;
    KLineEdit* classNameEdit;
    QRadioButton* useButton;
    QLabel* fileNameLabel;
    KLineEdit* fileNameEdit;

protected:
    QGridLayout* CreateImplemenationWidgetBaseLayout;
    QHBoxLayout* layout4;
    QSpacerItem* spacer4;
    QGridLayout* buttonGroup1Layout;
    QVBoxLayout* layout2;
    QVBoxLayout* layout2_2;

protected slots:
    virtual void languageChange();

    virtual void classNameChanged(const QString &);


};

#endif // CREATEIMPLEMENATIONWIDGETBASE_H
