/****************************************************************************
** Form interface generated from reading ui file './environmentvariableswidgetbase.ui'
**
** Created: Wed Apr 30 16:03:17 2008
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef ENVIRONMENTVARIABLESWIDGETBASE_H
#define ENVIRONMENTVARIABLESWIDGETBASE_H

#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QListView;
class QListViewItem;
class QPushButton;

class EnvironmentVariablesWidgetBase : public QWidget
{
    Q_OBJECT

public:
    EnvironmentVariablesWidgetBase( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~EnvironmentVariablesWidgetBase();

    QListView* listview;
    QPushButton* removevar_button;
    QPushButton* editvar_button;
    QPushButton* addvar_button;
    QPushButton* environmentButton;

public slots:
    virtual void addVarClicked();
    virtual void removeVarClicked();
    virtual void editVarClicked();
    virtual void environmentClicked();

protected:
    QGridLayout* EnvironmentVariablesWidgetBaseLayout;
    QSpacerItem* Spacer12;
    QSpacerItem* spacer2;

protected slots:
    virtual void languageChange();

};

#endif // ENVIRONMENTVARIABLESWIDGETBASE_H
