<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KDevTrollProject" version="3">
<MenuBar>
  <Menu name="build" >
    <Action name="build_build_project" />
    <Action name="build_install_project" />
    <Action name="build_rebuild_project" />
    <Action name="build_execute_project" />
    <Action name="build_clean_project" />
    <Action name="build_distclean_project" />
    <Separator/>
    <Action name="build_build_target" />
    <Action name="build_install_target" />
    <Action name="build_rebuild_target" />
    <Action name="build_execute_target" />
    <Action name="build_clean_target" />
    <Action name="build_distclean_target" />
    <Separator/>
    <Action name="build_compilefile" />
  </Menu>
</MenuBar>
<ToolBar name="buildToolBar" noMerge="1">
    <Action name="build_build_project" group="build_operations" />
    <Action name="build_rebuild_project" group="build_operations" />
    <Action name="build_execute_project" group="build_operations" />
    <Separator  group="build_operations"/>
    <Action name="build_build_target" group="build_operations" />
    <Action name="build_rebuild_target" group="build_operations" />
    <Action name="build_execute_target" group="build_operations" />
    <Separator  group="build_operations"/>
    <Action name="build_compilefile" group="build_operations" />
</ToolBar>
</kpartgui>
