<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kdevdesigner_shell" version="2">
<MenuBar>
  <Menu noMerge="1" name="file"><text>&amp;File</text>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>
  <Menu noMerge="1" name="edit"><text>&amp;Edit</text>
    <Merge/>
  </Menu>
  <Menu noMerge="1" name="project"><text>&amp;Project</text>
    <Merge/>
  </Menu>
  <Menu noMerge="1" name="tools"><text>&amp;Tools</text>
    <Merge/>
  </Menu>
  <Menu noMerge="1" name="layout"><text>&amp;Layout</text>
    <Merge/>
  </Menu>
  <Menu noMerge="1" name="window"><text>&amp;Window</text>
    <Merge/>
  </Menu>
  <Merge/>
  <Menu noMerge="1" name="settings"><text>&amp;Settings</text>
    <Action name="options_show_toolbar"/>
    <Action name="options_show_statusbar"/>
    <Merge name="show_merge"/>
    <Separator/>
    <Action name="options_configure_keybinding"/>
    <Action name="options_configure_toolbars"/>
    <Action name="options_configure"/>
    <Merge name="configure_merge"/>
    <Separator/>
  </Menu>
</MenuBar>
</kpartgui>
