

#include "$NEWFILENAMELC$.h"

$NEWCLASS$::$NEWCLASS$(QWidget* parent, Qt::WFlags fl)
: $QTBASECLASS$( parent, fl ), Ui::$BASECLASS$()
{
	setupUi(this);
}

$NEWCLASS$::~$NEWCLASS$()
{
}

/*$SPECIALIZATION$*/

