<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KDevClassView" version="10">
<MenuBar>
	<Menu name="project" >
		<Action name="inheritance_dia" group="project_classes" />
	</Menu>
</MenuBar>
<Menu name="kdevhidden" >
	<Action name="navigator_jump_to_next_function"/>
	<Action name="navigator_jump_to_previous_function"/>
</Menu>

<ToolBar name="browserToolBar">
  <text>Browser Toolbar</text>
  <Separator/>
  <Action name="classview_sync_with_editor" />
  <Action name="functionsnav_combo" />
</ToolBar>
</kpartgui>

