/****************************************************************************
** Form interface generated from reading ui file './removesubprojectdlgbase.ui'
**
** Created: Wed Apr 30 16:03:18 2008
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef REMOVESUBPROJECTDLGBASE_H
#define REMOVESUBPROJECTDLGBASE_H

#include <qvariant.h>
#include <qdialog.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QGroupBox;
class QLabel;
class QCheckBox;
class QPushButton;

class RemoveSubprojectDlgBase : public QDialog
{
    Q_OBJECT

public:
    RemoveSubprojectDlgBase( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~RemoveSubprojectDlgBase();

    QGroupBox* fileGroupBox;
    QLabel* removeLabel;
    QCheckBox* removeCheckBox;
    QLabel* noticeLabel;
    QPushButton* removeButton;
    QPushButton* cancelButton;

public slots:
    virtual void accept();
    virtual void reject();

protected:
    QVBoxLayout* RemoveSubprojectDlgBaseLayout;
    QVBoxLayout* fileGroupBoxLayout;
    QHBoxLayout* buttonLayout;
    QSpacerItem* buttonSpacer;

protected slots:
    virtual void languageChange();

};

#endif // REMOVESUBPROJECTDLGBASE_H
