/****************************************************************************
** Form interface generated from reading ui file './runoptionswidgetbase.ui'
**
** Created: Wed Apr 30 16:03:17 2008
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef RUN_OPTIONS_WIDGET_H
#define RUN_OPTIONS_WIDGET_H

#include <qvariant.h>
#include <qwidget.h>
#include <ksqueezedtextlabel.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QGroupBox;
class QLabel;
class KLineEdit;
class KURLRequester;
class QCheckBox;

class RunOptionsWidgetBase : public QWidget
{
    Q_OBJECT

public:
    RunOptionsWidgetBase( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~RunOptionsWidgetBase();

    QGroupBox* mainProgramGroupBox;
    QLabel* notelabel;
    KLineEdit* runargs_edit;
    QLabel* mainprogram_label_2;
    QLabel* mainprogram_label;
    KURLRequester* mainprogram_edit;
    QLabel* mainprogram_label_3;
    KLineEdit* debugargs_edit;
    QLabel* mainprogram_label_3_2;
    KURLRequester* cwd_edit;
    QGroupBox* groupBox2;
    QCheckBox* autocompile_box;
    QCheckBox* autoinstall_box;
    QCheckBox* autokdesu_box;
    QCheckBox* startinterminal_box;
    QGroupBox* env_var_group;

public slots:
    virtual void mainProgramChanged();

protected:
    QVBoxLayout* run_options_widgetLayout;
    QGridLayout* mainProgramGroupBoxLayout;
    QVBoxLayout* groupBox2Layout;

protected slots:
    virtual void languageChange();

};

#endif // RUN_OPTIONS_WIDGET_H
