/****************************************************************************
** Form interface generated from reading ui file './settingswidget.ui'
**
** Created: Wed Apr 30 15:44:21 2008
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef FORM2_H
#define FORM2_H

#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QCheckBox;
class QLabel;
class QComboBox;
class KURLRequester;
class KFontRequester;
class QButtonGroup;
class QRadioButton;
class KLineEdit;

class SettingsWidget : public QWidget
{
    Q_OBJECT

public:
    SettingsWidget( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~SettingsWidget();

    QCheckBox* lastProjectCheckbox;
    QLabel* textLabel2;
    QCheckBox* lineWrappingCheckBox;
    QLabel* compilerLabel;
    QComboBox* compileOutputCombo;
    QLabel* textLabel2_2;
    QLabel* textLabel1;
    KURLRequester* projectsURL;
    QLabel* buttonLabel;
    KFontRequester* outputFont;
    QButtonGroup* terminalButtonGroup;
    QRadioButton* useKDETerminal;
    QRadioButton* useOtherTerminal;
    KLineEdit* terminalEdit;
    QLabel* textLabel5_2;
    QLabel* textLabel5;
    QButtonGroup* designerButtonGroup;
    QRadioButton* embeddedDesignerRadioButton;
    QRadioButton* seperateAppRadioButton;
    QRadioButton* qtDesignerRadioButton;
    QCheckBox* dirNavigMsgCheckBox;
    QButtonGroup* buttonGroup3;
    QRadioButton* forceCLocaleRadio;
    QRadioButton* userLocaleRadio;

protected:
    QGridLayout* Form2Layout;
    QSpacerItem* spacer13;
    QSpacerItem* spacer13_2_2;
    QSpacerItem* spacer13_2_2_2;
    QSpacerItem* spacer23;
    QSpacerItem* spacer13_2;
    QHBoxLayout* layout10;
    QHBoxLayout* layout5;
    QHBoxLayout* layout4;
    QGridLayout* terminalButtonGroupLayout;
    QHBoxLayout* layout9;
    QGridLayout* designerButtonGroupLayout;
    QVBoxLayout* buttonGroup3Layout;

protected slots:
    virtual void languageChange();

};

#endif // FORM2_H
