<!DOCTYPE kpartgui>
<kpartplugin name="valgrind" library="libvalgrindplugin" version="3">
<MenuBar>
 <Menu name="debug"><Text>&amp;Debug</Text>
  <Action name="tools_valgrind" group="profile"/>
  <Action name="tools_calltree" group="profile"/>
 </Menu>
</MenuBar>
</kpartplugin>
