/****************************************************************************
** Form interface generated from reading ui file './generalinfowidgetbase.ui'
**
** Created: Wed Apr 30 15:44:20 2008
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef GENERAL_INFO_WIDGET_H
#define GENERAL_INFO_WIDGET_H

#include <qvariant.h>
#include <qpixmap.h>
#include <qwidget.h>
#include <ksqueezedtextlabel.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QComboBox;
class QLabel;
class KLineEdit;
class KSqueezedTextLabel;
class QTextEdit;

class GeneralInfoWidgetBase : public QWidget
{
    Q_OBJECT

public:
    GeneralInfoWidgetBase( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~GeneralInfoWidgetBase();

    QComboBox* project_directory_combo;
    QLabel* description_label;
    QLabel* project_directory_label;
    QLabel* author_label;
    QLabel* email_label;
    QLabel* version_label;
    KLineEdit* version_edit;
    KLineEdit* email_edit;
    KLineEdit* author_edit;
    QLabel* project_directory_diagnostic_icon;
    KLineEdit* project_directory_edit;
    KSqueezedTextLabel* project_directory_diagnostic_label;
    QLabel* textLabel1;
    QComboBox* encoding_combo;
    QTextEdit* description_edit;

protected:
    QGridLayout* general_info_widgetLayout;
    QSpacerItem* spacer1;

protected slots:
    virtual void languageChange();

private:
    QPixmap image0;

};

#endif // GENERAL_INFO_WIDGET_H
