<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KDevAStyle" version="6">
<MenuBar>
 <Menu name="edit">
  <Action name="edit_astyle" group="edit_astyle"/>
 </Menu>
 <Menu name="tools" >
   <Action name="tools_astyle" group="tools_astyle"/>
 </Menu>
</MenuBar>
<ToolBar name="astyleToolBar">
  <text>Astyle Toolbar</text>
  <Action name="tools_astyle" group="tools_astyle"/>
  <Action name="edit_astyle" group="edit_astyle"/>
</ToolBar>

</kpartgui>
