<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KDevBashSupport" version="2">
<MenuBar>
  <Menu name="build" >
    <Action name="build_execute" />
  </Menu>
</MenuBar>
<ToolBar name="buildToolBar">
    <Action name="build_execute" group="build_operations" />
</ToolBar>
</kpartgui>
